"""
ueditor 管理面板模块
@author Peter
@time 20180814
"""

from __future__ import absolute_import
from django.db import models
from django.contrib.admin.widgets import AdminTextareaWidget
from .widgets import UEditor, AdminUEditor

__all__ = ['UEditorField', 'UEditor']


class UEditorField(models.TextField):
    """
    A text area model field for HTML content.

    It uses the TinyMCE 4 widget in forms.

    Example::

        from django.db.models import Model
        from tinymce import HTMLField

        class Foo(Model):
            html_content = HTMLField('HTML content')
    """

    def __init__(self, *args, **kwargs):
        self.ueditor_profile = kwargs.pop('profile', None)
        super(UEditorField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'widget': UEditor(profile=self.ueditor_profile)
        }
        defaults.update(kwargs)
        # As an ugly hack, we override the admin widget
        if defaults['widget'] == AdminTextareaWidget:
            defaults['widget'] = AdminUEditor(profile=self.ueditor_profile)
        return super(UEditorField, self).formfield(**defaults)



# -*- coding:utf-8 -*-
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.conf import settings
from datetime import *
import os, codecs
import json
import random
import re

config_path = os.path.join(os.path.dirname(__file__), "ueconfig.json")
base_dir = settings.BASE_DIR  # 项目的根目录


# 本地上传图片时构造json返回值
class JsonResult(object):
    def __init__(self, state="未知错误", url="", title="", original="", error="null"):
        super(JsonResult, self).__init__()
        self.state = state
        self.url = url
        self.title = title
        self.original = original
        self.error = error


# 构造返回json
def buildJsonResult(result):
    jsondata = {"state": result.state, "url": result.url, "title": result.title, "original": result.original,
                "error": result.error}
    return json.dumps(jsondata)


def buildFileName(pathformat, filename):
    """
        PathFormat处理
    """

    dt = datetime.now()
    name, ext = os.path.splitext(filename)

    # 创建替换字典
    keys = ['{filename}', '{time}', '{yyyy}', '{yy}', '{mm}', '{dd}', '{hh}', '{ii}', '{ss}', ]
    values = [name, '%H%M%S', '%Y', '%y', '%m', '%d', '%H', '%M', '%S', ]
    texts = dict(zip(keys, values))

    # 遍历对应替换
    format_text = pathformat
    for key, value in texts.items():
        format_text = format_text.replace(key, value)

    # 处理随机数
    regstr = r'{rand:(\d+?)}'
    ms = re.search(regstr, format_text)
    group = ms.group()
    if group:
        rand_length = int(ms.groups()[0])  # 获取随机数字的长度
        rand_number = random.randint(1, 10 ** rand_length - 1)  # 生成随机数
        rand_number = str(rand_number).zfill(rand_length)  # 不足位数补0
        format_text = format_text.replace(group, rand_number)

    return dt.strftime(format_text) + ext


# 读取json文件
def getConfigContent():
    jsonfile = open(config_path)
    content = json.load(jsonfile)
    return content


# 上传配置类
class UploadConfig(object):
    def __init__(self, PathFormat, UploadFieldName, SizeLimit, AllowExtensions, SavePath, Base64, Base64Filename):
        super(UploadConfig, self).__init__()
        self.PathFormat = PathFormat
        self.UploadFieldName = UploadFieldName
        self.SizeLimit = SizeLimit
        self.AllowExtensions = AllowExtensions
        self.SavePath = SavePath
        self.Base64 = Base64
        self.Base64Filename = Base64Filename


# 获取json配置中的某属性值
def GetConfigValue(key):
    config = getConfigContent()
    return config[key]


# 检查文件扩展名是否在允许的扩展名内
def CheckFileType(filename, AllowExtensions):
    exts = list(AllowExtensions)
    name, ext = os.path.splitext(filename)
    return ext in exts


def CheckFileSize(filesize, SizeLimit):
    return filesize < SizeLimit


# 处理上传图片、文件、视频文件
@csrf_exempt
def uploadFile(request, config):
    result = JsonResult()
    if config.Base64:
        pass
    else:
        buf = request.FILES.get(config.UploadFieldName)
        filename = buf.name
        if not CheckFileType(filename, config.AllowExtensions):
            result.error = u"不允许的文件格式"
            return HttpResponse(buildJsonResult(result))

        if not CheckFileSize(buf.size, config.SizeLimit):
            result.error = u"文件大小超出服务器限制"
            return HttpResponse(buildJsonResult(result))

        try:
            truelyName = buildFileName(config.PathFormat, filename)
            webUrl = config.SavePath + truelyName
            savePath = base_dir + webUrl

            # 判断文件夹是否存在，不存在则创建
            folder, filename = os.path.split(savePath)
            if not os.path.isdir(folder):
                os.makedirs(folder)
            print(base_dir, savePath)

            f = codecs.open(savePath, "wb")
            for chunk in buf.chunks():
                f.write(chunk)
            f.flush()
            f.close()

            add_watermark(savePath)  # 加水印

            result.state = "SUCCESS"
            result.url = truelyName
            result.title = truelyName
            result.original = truelyName
            response = HttpResponse(buildJsonResult(result))
            response["Content-Type"] = "text/plain"
            return response
        except Exception as e:
            result.error = u"网络错误"
            return HttpResponse(buildJsonResult(result))


# 加水印
def add_watermark(savePath):
    try:
        # 判断是否是图片文件
        if not os.path.splitext(savePath)[-1].lower() in ['.jpg', '.jpge', '.png', '.bmp']:
            return

        # 获取配置
        config = getConfigContent()
        is_mark = config.get('openWaterMark', False)  # 是否开启加水印功能
        watermark = config.get('waterMarkText', '')  # 水印内容
        font = config.get('waterMarkFont', 'msyhbd.ttf')  # 字体
        size = config.get('waterMarkSize', 15)  # 字体大小
        bottom = config.get('waterMarkBottom', 45)  # 下边距
        right = config.get('waterMarkRight', 155)  # 右边距

        # 判断是否开启了加水印功能
        if not is_mark:
            return

        # python2.7 pillow
        from PIL import Image, ImageDraw, ImageFont

        # 打开图片
        im = Image.open(savePath).convert('RGBA')

        # 透明的图层，用于写文本
        text_layer = Image.new('RGBA', im.size, (0, 0, 0, 0))
        draw = ImageDraw.Draw(text_layer)

        # 加载字体，设置大小
        font_path = os.path.join(os.path.dirname(__file__), font)
        fnt = ImageFont.truetype(font_path, size)  # 要加中文字体才能识别中文

        point = (text_layer.size[0] - right, text_layer.size[1] - bottom)  # 位置
        draw.text(point, watermark, font=fnt, fill=(255, 255, 255, 255))
        # out=Image.alpha_composite(im, text_layer)
        out = Image.composite(text_layer, im, text_layer)
        out.save(savePath)
        out.close()
    except Exception as e:
        print('[error]', e.message)


# 处理在线图片与在线文件
# 返回的数据格式：{"state":"SUCCESS","list":[{"url":"upload/image/20140627/6353948647502438222009315.png"},{"url":"upload/image/20140627/6353948659383617789875352.png"},{"url":"upload/image/20140701/6353980733328090063690725.png"},{"url":"upload/image/20140701/6353980745691597223366891.png"},{"url":"upload/image/20140701/6353980747586705613811538.png"},{"url":"upload/image/20140701/6353980823509548151892908.png"}],"start":0,"size":20,"total":6}
def listFileManage(request, imageManagerListPath, imageManagerAllowFiles, listsize):
    pstart = request.GET.get("start")
    start = pstart == None and int(pstart) or 0
    psize = request.GET.get("size")
    size = psize == None and int(GetConfigValue(listsize)) or int(psize)
    localPath = base_dir + imageManagerListPath

    # 2016-10-31自动创建在线管理的文件夹
    if not os.path.isdir(localPath):
        os.makedirs(localPath)

    filelist = []
    exts = list(imageManagerAllowFiles)
    index = start
    for imagename in os.listdir(localPath):
        name, ext = os.path.splitext(imagename)
        if ext in exts:
            filelist.append(dict(url=imagename))
            index += 1
            if index - start >= size:
                break
    jsondata = {"state": "SUCCESS", "list": filelist, "start": start, "size": size, "total": index}
    return HttpResponse(json.dumps(jsondata))


# 返回配置信息
def configHandler(request):
    content = getConfigContent()
    callback = request.GET.get("callback")
    if callback:
        return HttpResponse("{0}{1}".format(callback, json.dumps(content)))
    return HttpResponse(json.dumps(content))


# 图片上传控制
@csrf_exempt
def uploadimageHandler(request):
    AllowExtensions = GetConfigValue("imageAllowFiles")
    PathFormat = GetConfigValue("imagePathFormat")
    SizeLimit = GetConfigValue("imageMaxSize")
    UploadFieldName = GetConfigValue("imageFieldName")
    SavePath = GetConfigValue("imageUrlPrefix")
    upconfig = UploadConfig(PathFormat, UploadFieldName, SizeLimit, AllowExtensions, SavePath, False, '')
    return uploadFile(request, upconfig)


def uploadvideoHandler(request):
    AllowExtensions = GetConfigValue("videoAllowFiles")
    PathFormat = GetConfigValue("videoPathFormat")
    SizeLimit = GetConfigValue("videoMaxSize")
    UploadFieldName = GetConfigValue("videoFieldName")
    SavePath = GetConfigValue("videoUrlPrefix")
    upconfig = UploadConfig(PathFormat, UploadFieldName, SizeLimit, AllowExtensions, SavePath, False, '')
    return uploadFile(request, upconfig)


def uploadfileHandler(request):
    AllowExtensions = GetConfigValue("fileAllowFiles")
    PathFormat = GetConfigValue("filePathFormat")
    SizeLimit = GetConfigValue("fileMaxSize")
    UploadFieldName = GetConfigValue("fileFieldName")
    SavePath = GetConfigValue("fileUrlPrefix")
    upconfig = UploadConfig(PathFormat, UploadFieldName, SizeLimit, AllowExtensions, SavePath, False, '')
    return uploadFile(request, upconfig)


# 在线图片
def listimageHandler(request):
    imageManagerListPath = GetConfigValue("imageManagerListPath")
    imageManagerAllowFiles = GetConfigValue("imageManagerAllowFiles")
    imagelistsize = GetConfigValue("imageManagerListSize")
    return listFileManage(request, imageManagerListPath, imageManagerAllowFiles, imagelistsize)


# 在线文件
def ListFileManagerHander(request):
    fileManagerListPath = GetConfigValue("fileManagerListPath")
    fileManagerAllowFiles = GetConfigValue("fileManagerAllowFiles")
    filelistsize = GetConfigValue("fileManagerListSize")
    return listFileManage(request, fileManagerListPath, fileManagerAllowFiles, filelistsize)


actions = {
    "config": configHandler,
    "uploadimage": uploadimageHandler,
    "uploadvideo": uploadvideoHandler,
    "uploadfile": uploadfileHandler,
    "listimage": listimageHandler,
    "listfile": ListFileManagerHander
}


@csrf_exempt
def handler(request):
    action = request.GET.get("action")
    return actions.get(action)(request)



# coding: utf-8
# License: MIT, see LICENSE.txt
"""
ueditor 4 forms widget

This ueditor widget was copied and extended from this code by John D'Agostino:
http://code.djangoproject.com/wiki/CustomWidgetsueditor
"""

from __future__ import unicode_literals
from __future__ import absolute_import
import json
import logging
import os
import sys
from django.conf import settings
from django.contrib.staticfiles import finders
from django.forms import Textarea, Media
from django.utils.encoding import smart_text
from django.utils.safestring import mark_safe
from django.utils.translation import get_language, get_language_bidi
from django.template.loader import render_to_string
from django.contrib.admin import widgets as admin_widgets
from . import settings as qtue_settings

# __all__ = ['UEditor', 'render_ueditor_init_js']
__all__ = ['UEditor']

logging.basicConfig(format='[%(asctime)s] %(module)s: %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)
logger.setLevel(20)

if sys.version_info[:2] < (3, 0):
    logger.warning(
        'Deprecation warning: Python 2 support will be removed '
        'in future releases of ueditor4-lite!'
    )


def language_file_exists(language_code):
    """
    Check if ueditor has a language file for the specified lang code

    :param language_code: language code
    :type language_code: str
    :return: check result
    :rtype: bool
    """
    filename = '{0}.js'.format(language_code)
    path = os.path.join('UE', 'lang', language_code, filename)
    return finders.find(path) is not None


def get_language_config():
    """
    Creates a language configuration for ueditor4 based on Django project settings

    :return: language- and locale-related parameters for ueditor 4
    :rtype: dict
    """
    language_code = convert_language_code(get_language() or settings.LANGUAGE_CODE)
    if not language_file_exists(language_code):
        language_code = language_code[:2]
        if not language_file_exists(language_code):
            # Fall back to English if Tiny MCE 4 does not have required translation
            language_code = 'en'
    config = {'language': language_code}
    if get_language_bidi():
        config['directionality'] = 'rtl'
    else:
        config['directionality'] = 'ltr'
    if qtue_settings.USE_SPELLCHECKER:
        try:
            from enchant import list_languages
        except ImportError as ex:
            raise ImportError(
                'To use spellchecker you need to install pyenchant first!'
            ).with_traceback(ex.__traceback__)
        enchant_languages = list_languages()
        if settings.DEBUG:
            logger.info('Enchant languages: {0}'.format(enchant_languages))
        lang_names = []
        for lang, name in settings.LANGUAGES:
            lang = convert_language_code(lang)
            if lang not in enchant_languages:
                lang = lang[:2]
            if lang not in enchant_languages:
                logger.error('Missing {0} spellchecker dictionary!'.format(lang))
                continue
            if config.get('spellchecker_language') is None:
                config['spellchecker_language'] = lang
            lang_names.append('{0}={1}'.format(name, lang))
        config['spellchecker_languages'] = ','.join(lang_names)
    return config


def convert_language_code(django_lang):
    """
    Converts Django language codes "ll-cc" into ISO codes "ll_CC" or "ll"

    :param django_lang: Django language code as ll-cc
    :type django_lang: str
    :return: ISO language code as ll_CC
    :rtype: str
    """
    lang_and_country = django_lang.split('-')
    try:
        return '_'.join((lang_and_country[0], lang_and_country[1].upper()))
    except IndexError:
        return lang_and_country[0]


def render_ueditor_init_js(qtue_config, callbacks, id_):
    """
    Renders ueditor.init() JavaScript code

    :param qtue_config: ueditor 4 configuration
    :type qtue_config: dict
    :param callbacks: ueditor callbacks
    :type callbacks: dict
    :param id_: HTML element's ID to which ueditor is attached.
    :type id_: str
    :return: ueditor.init() code
    :rtype: str
    """
    if qtue_settings.USE_FILEBROWSER and 'file_browser_callback' not in callbacks:
        callbacks['file_browser_callback'] = 'djangoFileBrowser'
    if qtue_settings.USE_SPELLCHECKER and 'spellchecker_callback' not in callbacks:
        callbacks['spellchecker_callback'] = 'ueditor4_spellcheck'
    if id_:
        qtue_config['selector'] = qtue_config.get('selector', 'textarea') + '#{0}'.format(id_)
    return render_to_string('ueditor/ueditor_init.js',
                            context={'callbacks': callbacks,
                                     'ueditor_config': json.dumps(qtue_config)[1:-1],
                                     'is_admin_inline': '__prefix__' in id_})


class UEditor(Textarea):
    """
    ueditor 4 widget

    It replaces a textarea form widget with a rich-text WYSIWYG `ueditor 4`_ editor widget.

    :param attrs: General Django widget attributes.
    :type attrs: dict
    :param ue_attrs: Additional configuration parameters for ueditor 4.
        They *amend* the existing configuration.
    :type ue_attrs: dict
    :param profile: ueditor 4 configuration parameters.
        They *replace* the existing configuration.
    :type profile: dict

    .. _ueditor 4: https://www.ueditor.com/
    """

    def __init__(self, attrs=None, ue_attrs=None, profile=None):
        super(UEditor, self).__init__(attrs)
        self.ue_attrs = ue_attrs or {}
        self.profile = {}
        default_profile = profile or qtue_settings.CONFIG.copy()
        self.profile.update(default_profile)

    def build_attrs(self, base_attrs, extra_attrs=None, **kwargs):
        attributes = dict(base_attrs, **kwargs)
        if extra_attrs:
            attributes.update(extra_attrs)
        return attributes

    def render(self, name, value, attrs=None, renderer=None):
        if value is None:
            value = ''
        value = smart_text(value)
        ue_html = """<script id="{name}" name="{name}" type="text/plain">
                                            </script>
                                            <textarea id="{name}_val" style="display:none;" >{value}</textarea>
                                            <script>
                                            (function ($) {{
                                                var ue = UE.getEditor("{name}",{{zIndex: 100}});
                                                ue.ready(function () {{
                                                    var content_val =$('#{name}_val').val();
                                                    ue.setContent(content_val);
                                                }});
                                            }})(grp.jQuery);
                                            </script>""".format(name=name, value=value)

        return mark_safe(ue_html)

    @property
    def media(self):
        js = [qtue_settings.CONFIG_URL, qtue_settings.JS_URL]
        return Media(js=js)


class AdminUEditor(UEditor, admin_widgets.AdminTextareaWidget):
    """ueditor 4 widget for Django Admin interface"""
    pass



# coding:utf-8
from django.conf.urls import include, url
from django.views.static import serve
from .controller import handler
import os

ueditor_path = os.path.join(os.path.dirname(__file__), "UE")

urlpatterns = [
    url(r'^$', handler),
    url(r'^UE/(?P<path>.*)$', serve, {'document_root': ueditor_path}),
]



# coding: utf-8
# License: MIT, see LICENSE.txt

from __future__ import absolute_import
from django.conf import settings
from django.contrib.staticfiles.storage import staticfiles_storage

DEFAULT = {}
CONFIG = getattr(settings, 'UEDITOR_DEFAULT_CONFIG', DEFAULT)
CONFIG_URL = staticfiles_storage.url('/ueditor/ueditor.config.js')
JS_URL = getattr(settings, 'UEDITOR_JS_URL', None)
if JS_URL is None:
    JS_URL = staticfiles_storage.url('ueditor/ueditor.all.min.js')




#!/usr/bin/env python

from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='django-ueditor',
    description=(
        'Qitian Inc. Django UEditor plugins. For quick use in Django Admin Panel'
    ),
    long_description=long_description,
    long_description_content_type="text/markdown",
    version='0.0.1',
    author='Peter Han',
    author_email='peter@qitian.com',
    url='https://www.qitian.biz/',
    license='MIT License',
    packages=find_packages(exclude=['tests', 'tests.*']),
    include_package_data=True,
    install_requires=[
        'django>=2.0'
    ],
    python_requires=">=3.6, !=3.0.*",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        'Topic :: Utilities',
    ],
)
